﻿using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.WebUtilities;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using MTWorkHR.Application.Identity;
using MTWorkHR.Application.Mapper;
using MTWorkHR.Application.Models;
using MTWorkHR.Core.Global;
using MTWorkHR.Core.IRepositories;
using MTWorkHR.Core.UnitOfWork;
using MTWorkHR.Infrastructure.Entities;
using MTWorkHR.Application.Services.Interfaces;
using MTWorkHR.Core.Email;
using MTWorkHR.Core.Entities;
using System.Linq.Dynamic.Core;
using System.Linq;
using Microsoft.AspNetCore.Http;
using MTWorkHR.Core.Entities.Base;

namespace MTWorkHR.Application.Services
{
    public class AttendanceService : BaseService<Attendance, AttendanceDto, AttendanceDto>, IAttendanceService
    {
        private readonly IUnitOfWork _unitOfWork;
        private readonly GlobalInfo _globalInfo;
        private readonly IUserService _userService;

        public AttendanceService(IUnitOfWork unitOfWork, GlobalInfo globalInfo, IUserService userService) :base(unitOfWork)
        {
            _unitOfWork = unitOfWork;
            _globalInfo = globalInfo;
            _userService = userService;
        }
        public async Task<PagingResultDto<AttendanceDto>> GetAll(AttendancePagingInputDto PagingInputDto)
        {
            var res = await _unitOfWork.Attendance.GetAllWithChildrenAsync();
            var query = res.Item1;

            if (_globalInfo.UserType != UserTypeEnum.Business)
            {
                query = query.Where(m => m.UserId == _globalInfo.UserId);

            }
            if (PagingInputDto.Filter != null)
            {
                var filter = PagingInputDto.Filter;
                query = query.Where(u => u.LeaveReason!.Contains(filter) || u.UserName!.Contains(filter));
            }
                  
            if (PagingInputDto.SearchDate != null)
            {
                query = query.Where(u => u.AttendanceDate.Date == PagingInputDto.SearchDate.Value.Date);
            }

            if (PagingInputDto.UserId != null)
            {
                query = query.Where(u => u.UserId == PagingInputDto.UserId);
            }

            var order = query.OrderBy(PagingInputDto.OrderByField + " " + PagingInputDto.OrderType);

            var page = order.Skip((PagingInputDto.PageNumber * PagingInputDto.PageSize) - PagingInputDto.PageSize).Take(PagingInputDto.PageSize);

            var total = await query.CountAsync();

            var list = MapperObject.Mapper
                .Map<IList<AttendanceDto>>(await page.ToListAsync());

           // var employeesListx = list.Select(a => a.UserId).Distinct();
            var employeesList = await _userService.GetAllCompanyEmployees();
            Dictionary<string, UserBasicInfoDto> employeeDictList = new Dictionary<string, UserBasicInfoDto>();
            List<AttendanceDto> attendanceResult = new List<AttendanceDto>();

            if (string.IsNullOrEmpty(PagingInputDto.UserId))
            {
                if (PagingInputDto.UserName != null)
                {
                    //employeesList = employeesList.Where(u => u.FirstName.Contains(PagingInputDto.UserName) || u.LastName.Contains(PagingInputDto.UserName) ||
                    //u.UserName.Contains(PagingInputDto.UserName) || u.Email.Contains(PagingInputDto.UserName)
                    //).ToList();
                    var filter = PagingInputDto.UserName;
                    var filters = filter.Split(" ");
                    var firstNameFilter = filters[0];
                    var lastNameFilter = filters.Length > 1 ? filters[1] : "";

                    employeesList = employeesList.Where(u =>
                        u.UserName.Contains(filter) || u.Email.Contains(filter) || u.FavoriteName.Contains(filter) || u.FirstName.Contains(filter) || u.LastName.Contains(filter) ||
                        u.FirstName.Contains(firstNameFilter) && (string.IsNullOrEmpty(lastNameFilter) || u.LastName.Contains(lastNameFilter)) ||
                        u.LastName.Contains(lastNameFilter) && (string.IsNullOrEmpty(firstNameFilter) || u.FirstName.Contains(firstNameFilter))
                    ).ToList();
                }
                foreach (var employee in employeesList)
                {
                    var latestAttendance = list.LastOrDefault(a => a.UserId == employee.Id);

                    var user = await _userService.GetUserWithAttachmentById(employee.Id);
                    if (user != null)
                    {
                        var entitiy = MapperObject.Mapper.Map<UserBasicInfoDto>(user);
                        //item.Employee = entitiy;
                        var attach = user.UserAttachments?.FirstOrDefault(a => a.AttachmentTypeId == 9);
                        if (attach != null)
                            using (var stream = new MemoryStream(attach.Content))
                            {
                                var file = new FormFile(stream, 0, stream.Length, Path.GetFileNameWithoutExtension(attach.FileName), attach.FilePath)
                                {
                                    Headers = new HeaderDictionary(),
                                    ContentType = attach.ContentType,
                                };

                                System.Net.Mime.ContentDisposition cd = new System.Net.Mime.ContentDisposition
                                {
                                    FileName = file.FileName
                                };
                                file.ContentDisposition = cd.ToString();
                                entitiy.ProfileImage = file;

                            }
                        employeeDictList.Add(employee.Id, entitiy);
                        if (latestAttendance != null)
                            latestAttendance.Employee = entitiy;
                        else
                        {
                            latestAttendance = new AttendanceDto { UserId = employee.Id, UserName = employee.UserName, Employee = entitiy };
                        }
                        attendanceResult.Add(latestAttendance);
                    }
                }
            }
            else
            {
                var user = await _userService.GetUserWithAttachmentById(PagingInputDto.UserId);
                if (user != null)
                {
                    var entitiy = MapperObject.Mapper.Map<UserBasicInfoDto>(user);
                    foreach (var item in list)
                    {
                        var attach = user.UserAttachments?.FirstOrDefault(a => a.AttachmentTypeId == 9);
                        if (attach != null)
                            using (var stream = new MemoryStream(attach.Content))
                            {
                                var file = new FormFile(stream, 0, stream.Length, Path.GetFileNameWithoutExtension(attach.FileName), attach.FilePath)
                                {
                                    Headers = new HeaderDictionary(),
                                    ContentType = attach.ContentType,
                                };

                                System.Net.Mime.ContentDisposition cd = new System.Net.Mime.ContentDisposition
                                {
                                    FileName = file.FileName
                                };
                                file.ContentDisposition = cd.ToString();
                                entitiy.ProfileImage = file;

                            }
                        item.Employee = entitiy;
                    }
                }
            }
                
            var response = new PagingResultDto<AttendanceDto>
            {
                Result = !string.IsNullOrEmpty(PagingInputDto.UserId) ? list : attendanceResult,
                //Result = ! string.IsNullOrEmpty( PagingInputDto.UserId ) ? list : dayGroupByUser,
                Total = !string.IsNullOrEmpty(PagingInputDto.UserId) ? total : attendanceResult.Count //total
            };

            return response;
        }

        public override async Task<AttendanceDto> Create(AttendanceDto input)
        {
            if (input.CheckInTime.HasValue)
                input.AttendanceDate = input.CheckInTime.Value.Date;
            var oldEntity = await _unitOfWork.Attendance.GetAttendanceByUserId(input.UserId, input.AttendanceDate);
            if (oldEntity is null)
            {
                var entitiy = MapperObject.Mapper.Map<Attendance>(input);
                if (entitiy is null)
                    throw new AppException(ExceptionEnum.MapperIssue);

                var newEntity = await _unitOfWork.Attendance.AddAsync(entitiy);
                var Success = await _unitOfWork.CompleteAsync();

                var response = Mapper.MapperObject.Mapper.Map<AttendanceDto>(newEntity);
                return response;
            }
            else
            {
                throw new AppException(ExceptionEnum.UserAlreadyCheckedIn);
                var response = Mapper.MapperObject.Mapper.Map<AttendanceDto>(oldEntity);
                return response;
            }

        }

        public override async Task<AttendanceDto> Update(AttendanceDto input)
        {
            Attendance? entity = null;
            if (input.CheckInTime.HasValue)
                input.AttendanceDate = input.CheckInTime.Value.Date;
            else if (input.CheckOutTime.HasValue)
                input.AttendanceDate = input.CheckOutTime.Value.Date;
            if (input.Id > 0)
                entity = await _unitOfWork.Attendance.GetByIdAsync(input.Id);
            else
                entity = await _unitOfWork.Attendance.GetAttendanceByUserId(input.UserId, input.AttendanceDate);
            if (entity is null)
                throw new AppException(ExceptionEnum.RecordNotExist);
            entity.CheckOutTime = input.CheckOutTime;
            entity.LeaveType = input.LeaveType;
            entity.LeaveReason = input.LeaveReason;
          
            await _unitOfWork.CompleteAsync();

            var response = Mapper.MapperObject.Mapper.Map<AttendanceDto>(entity);
            return response;
        }

        public async Task<AttendanceDto?> GetAttendanceByUserId(string userId, DateTime attendanceDate)
        {
            var entity = await _unitOfWork.Attendance.GetAttendanceByUserId(userId, attendanceDate);
            if (entity is null)
                return null;
            var response = Mapper.MapperObject.Mapper.Map<AttendanceDto>(entity);
            return response;
        }

    }
}
